# encoding: utf-8

require_relative 'app_helper'
require_relative 'minify_resources'

class MyApp < Sinatra::Application
  enable :sessions

  # --- config

  configure do
    use Rack::Session::Cookie, secret:'gazelle', old_secret:'gazelle-old'
    use Rack::Flash, sweep:true
    set :haml, format: :html5
  end

  configure :development do
    set :show_exceptions, true
    set :css_files, MinifyResources::get_original_css_files
    set :js_files,  MinifyResources::get_original_js_files
  end

  configure :production do
    set :haml, { ugly: true }
    set :clean_trace, true
    set :show_exceptions, false
    MinifyResources.minify_all
    set :css_files, MinifyResources::get_minified_css_files
    set :js_files,  MinifyResources::get_minified_js_files
  end

  # --- helpers

  helpers do
    include Rack::Utils
    alias_method :h, :escape_html
  end

  # --- errors

  error do
    content_type :json

    sinatra_error = env['sinatra.error']

    error_code = 500
    error_code = 400 if sinatra_error.class == ArgumentError

    output = {
      status: 'error',
      executed_at: Time.now.strftime('%Y-%m-%d %H:%M:%S'),
      status_message: sinatra_error.message
    }
    output[:backtrace] = sinatra_error.backtrace if ['test', 'development'].include?(ENV['RACK_ENV'])

    halt error_code, JSON.pretty_generate(output)
  end

  not_found do
    # TODO
  end

end

require_relative 'helpers/init'
require_relative 'models/init'
require_relative 'routes/init'

# vim:ff=unix ts=2 ss=2
