# encoding: utf-8

ENV['RACK_ENV'] ||= 'development'

require 'data_mapper'
require 'yaml'

require_relative 'status'
require_relative 'ats'
require_relative 'candidate'
require_relative 'state'
require_relative 'email'
require_relative 'invitation'
require_relative 'test'
require_relative 'token'
require_relative 'user'

database_config = YAML::load(File.read(File.expand_path('../../config/database.yml', __FILE__)))

configure :development do
  adapter = database_config['development']['adapter']
  host = database_config['development']['host']
  port = database_config['development']['port']
  username = database_config['development']['username']
  password = database_config['development']['password']
  database = database_config['development']['database']
  loglevel = database_config['development']['log_level']

  DataMapper::Logger.new($stdout, loglevel.to_sym)
  DataMapper.setup(:default, "#{adapter}://#{username}:#{password}@#{host}:#{port}/#{database}")
  DataMapper.finalize
  DataMapper.auto_upgrade!
end

configure :production do
  adapter = database_config['production']['adapter']
  host = database_config['production']['host']
  port = database_config['production']['port']
  username = database_config['production']['username']
  password = database_config['production']['password']
  database = database_config['production']['database']
  log = database_config['production']['log']
  loglevel = database_config['production']['log_level']
  DataMapper::Logger.new(log, loglevel.to_sym) unless log.nil?
  DataMapper.setup(:default, "#{adapter}://#{username}:#{password}@#{host}:#{port}/#{database}")
  DataMapper.finalize
  DataMapper.auto_upgrade!
end

# vim:ff=unix ts=2 ss=2
