# encoding: utf-8

require 'open-uri'
require 'json'
require 'uri'
require 'net/http'

class MyApp < Sinatra::Application

  hacker_rank_url = YAML::load(File.read(File.expand_path('../../config/hackerrankpaths.yml', __FILE__)))
  load_test_url_o = hacker_rank_url['load_test']
  load_candidate_url_o = hacker_rank_url['getUsers']
  load_result_url_o = hacker_rank_url['getComments']

  #function which will download all data from hackerrank and then it will compare with all data from database and update them
  post '/load_contents' do
    data = JSON.parse(request.body.read, :symbolize_names => true)
    load_comments =  to_boolean(data[:loadComments])
    all_candidates = is_database_empty?

    t = Token.last(:fields => [:token])
    token = t.token

    load_candidate_url = load_candidate_url_o.clone
    load_result_url = load_result_url_o.clone
    load_test_url = load_test_url_o.clone

    load_candidate_url.gsub!('{token}', token)
    load_result_url.gsub!('{token}', token)
    load_test_url = load_test_url.gsub!('{token}', token)
    #loading and saving tests
    test_buffer = open(load_test_url).read
    result = JSON.parse(test_buffer)

    save_tests(result)

    tests = Test.all()
    tests.each do |t|
      load_candidate = load_candidate_url.clone
      load_candidate = load_candidate.gsub!('{idTest}', t.id.to_s)
      candidate_buffer = open(load_candidate).read
      candidates = JSON.parse(candidate_buffer)
      candidates['data'].each do |c|
        comment = ''
        if c['id'] && load_comments
          l = load_result_url.clone
          l = l.gsub!('{idTest}', t['id'].to_s)
          l = l.gsub!('{idUser}', c['id'].to_s)

          result_buffer = open(l).read
          result = JSON.parse(result_buffer)
          #we try if he has result comment
          comment = comment_is_not_null?(result)
        end
        user_status = State.get(c['status'])
        ats_status = c['ats_state'] ? Ats.get(c['ats_state']) : Ats.get(0)
        date_of_invite = date_is_not_null?(c['invite_details'])
        date_when_test_written =  c['endtime'] ? Date.parse(c['endtime']) : nil
        status = c['ats_state'] ? Status.get(c['ats_state']) : Status.get(0)
        #we create candidate and save him
        can = Candidate.new(:email => c['email'],
                        :name => c['name'] ? c['name'] : 'Nezadáno',
                        :date_of_invite => date_of_invite ? date_of_invite : nil,
                        :date_when_test_written => c['endtime'] ? c['endtime'] : nil,
                        :test_result => comment,
                        :test_link => c['link'] ? c['link'] : nil,
                        :report_link => c['report_url'],
                        :password_to_test => nil,
                        :note1 => "",
                        :note2 => "",
                        :test => t,
                        :status => status,
                        :ats => ats_status,
                        :state => user_status,
                        :date_of_prescreening => nil,
                        :date_of_interview => nil,
                        :hacker_rank_id => c['id'] ? c['id'] : -1)
        if all_candidates.nil?
          save_data(can)
        else
          update_data(can, all_candidates, load_comments)
        end
      end
    end
    make_invitation
    puts "Downloading of data completed"
  end

  #function which will return all tests
  get '/load_tests' do
    user = current_user
    if !user.nil?
      content_type :json
      tests = Test.all
      if !(tests.nil?)
        tests = tests.to_json
      end
      return tests
    end
  end

  #function which will return all candidates
  get '/load_candidates' do
    user = current_user
    if !user.nil?
      content_type :json
      candidates = Candidate.all()
      if !(candidates.nil?)
        candidates = candidates.to_json
      end
      return candidates
    end
  end

  #function which will return all ats statuses
  get '/load_ats' do
    user = current_user
    if !user.nil?
      content_type :json
      ats = Ats.all()
      if !(ats.nil?)
        ats = ats.to_json
      end
      return ats
    end
  end

  #function which will return all email templates
  get '/load_email_template' do
    user = current_user
    if !user.nil?
      content_type :json
      emails = Email.all()
      if !(emails.nil?)
        emails = emails.to_json
      end
      return emails
    end
  end

  #function which will insert invite in database
  post '/insert_invite' do
    user = current_user
    if !user.nil?
      data = JSON.parse(request.body.read, :symbolize_names => true)
      id_test = data[:id_test]
      email = data[:email]
      date = data[:date]
      id_email_template = data[:id_email_template]

      candidate = get_candidate(id_test, email)
      email_template = Email.get(id_email_template)
      save_invitation(date, candidate, email_template)
    end
  end

  #function which will return all invites for candidates
  get '/load_invites' do
    user = current_user
    if !user.nil?
      content_type :json
      invites = Invitation.all
      if !(invites.nil?)
        invites = invites.to_json
      end
      return invites
    end
  end

  #function which fetch all data, which are need for system
  get '/fetch' do
    fetch_user_status
    fetch_status
    fetch_ats
    fetch_token
  end

  #function which will fetch main email templates
  get '/fetch_templates' do
    fetch_email_template
  end

  #function which will return all statuses
  get '/load_status' do
    user = current_user
    if !user.nil?
      content_type :json
      status = Status.all
      if !(status.nil?)
        status = status.to_json
      end
      return status
    end
  end

  #function which will set ats statuse at hackerrank server (it is not used anywhere right now)
  post '/set_solved' do
    user = current_user
    if !user.nil?
      data = JSON.parse(request.body.read, :symbolize_names => true)
      id_test = data[:id_test]
      username = data[:username]
      ats_state = data[:ats_state]
      token = Token.last(:fields => [:token])
      header = {'Content-Type' => 'text/json'}
      url = "https://www.hackerrank.com/x/api/v2/tests/#{id_test}/candidates/set_ats_state"
      params = {:username => username, :ats_state => ats_state, :access_token => token.token}

      uri = URI.parse(url)
      http = Net::HTTP.new(uri.host, uri.port)
      http.use_ssl = true
      http.verify_mode = OpenSSL::SSL::VERIFY_NONE
      request = Net::HTTP::Put.new(uri.path)
      request.add_field('Content-Type', 'application/json')
      request.body = params.to_json

      response = http.request(request)

      candidate = get_candidate(id_test, username)
      candidate.update(:ats => Ats.get(ats_state))
    end
  end

  #function for updating informations about candidate
  post '/save_informations' do
    user = current_user
    if !user.nil?
      data = JSON.parse(request.body.read, :symbolize_names => true)
      id_test = data[:id_test].to_i
      username = data[:username]
      not1 = data[:note1]
      not2 = data[:note2]
      name = data[:name]
      status = Status.get(data[:status])
      date_interview = date_is_empty?(data[:date_of_interview])
      date_prescreening = date_is_empty?(data[:date_of_prescreening])
      candidate = get_candidate(id_test, username)

      if date_interview.nil?
        candidate[0].update(:date_of_interview => nil)
      else
        if date_interview.empty?
          candidate[0].update(:date_of_interview => nil)
        else
          date_interview = Date.parse date_interview
          candidate[0].update(:date_of_interview => date_interview)
        end
      end

      if date_prescreening.nil?
        candidate[0].update(:date_of_prescreening => nil)
      else
        if date_prescreening.empty?
          candidate[0].update(:date_of_prescreening => nil)
        else
          date_prescreening = Date.parse date_prescreening
          candidate[0].update(:date_of_prescreening => date_prescreening)
        end
      end
      candidate[0].update(:name => name)
      candidate[0].update(:note1 => not1)
      candidate[0].update(:note2 => not2)
      candidate[0].update(:status => status)
    end
  end

  #function crating candidate in database and it will create a new test invitation at hackerranks servers
  post '/send_invite' do
    user = current_user
    if !user.nil?
      data = JSON.parse(request.body.read, :symbolize_names => true)
      id_test = data[:id_test]
      email = data[:username]
      name = data[:name]
      note1 = data[:note1]
      note2 = data[:note2]
      force = data[:force]
      send_email = data[:send_email]
      hide_login_credentials = data[:hide_login_credentials]
      send_test = to_boolean(data[:send_test])
      email_template_id = data[:email_template]
      test_o = Test.first(:id => id_test)
      status = Status.first(:id => -1)
      user_exist = Candidate.all(:email => email) & Candidate.all(:status => status) & Candidate.all(:test => test_o)
      password_to_test = nil
      response_body = nil
      can = nil
      test_link = nil

      if send_test
        token = Token.last(:fields => [:token])
        params = {'username' => email, 'send_email' => "#{send_email}", 'force' => "#{force}", 'hide_login_credentials' => "#{hide_login_credentials}", 'access_token' => token.token}

        url = "https://www.hackerrank.com/x/api/v2/tests/#{id_test}/candidates"
        # Create the HTTP objects
        uri = URI.parse(url)
        http = Net::HTTP.new(uri.host, uri.port)
        http.use_ssl = true
        http.verify_mode = OpenSSL::SSL::VERIFY_NONE
        request = Net::HTTP::Post.new(uri.path)
        request.add_field('Content-Type', 'application/json')
        request.body = params.to_json

        response = http.request(request)
        response_body = JSON.parse(response.body)
        password_to_test = response_body['data']['password']
        test_link = response_body['data']['test_link']
      end

      if send_test
        status = Status.get(0)
      else
        status = Status.get(-1)
      end

      if user_exist[0].nil?
        can = Candidate.create(:email => email,
                      :name => name,
                      :date_of_invite => Date.today,
                      :date_when_test_written => nil,
                      :test_result => 'Nezadáno',
                      :test_link => test_link,
                      :report_link => nil,
                      :password_to_test => password_to_test,
                      :note1 => note1,
                      :note2 => note2,
                      :test => test_o,
                      :status => status,
                      :ats => Ats.get(0),
                      :state => State.get(-1),
                      :date_of_prescreening => nil,
                      :date_of_interview => nil,
                      :hacker_rank_id => nil)
      end

      if send_test && !user_exist[0].nil?
        can = user_exist[0]
        user_exist[0].update(:test => test_o)
        user_exist[0].update(:test_link => response_body['data']['test_link'])
        user_exist[0].update(:password_to_test => password_to_test)
      end

      email_template = Email.first(:id => 1)
      add_to_invite = Invitation.new(:date_send => can.date_of_invite,
                              :candidate => can,
                              :email => email_template,
                              :test => can.test)
      if !add_to_invite.save
        add_to_invite.errors.each do |error|
          puts error
        end
      end
      if send_test
        return response.body
      end
      return '{"data": {"test_link": "Nezadano"}}'
    end
  end


  #function for creating new email template
  post '/save_email_template' do
    user = current_user
    if !user.nil?
      data = JSON.parse(request.body.read, :symbolize_names => true)
      name_of_template = data[:name_of_template]
      subject = data[:subject]
      text = data[:text]
      id_template = data[:id]
      if id_template == -1
        new_email = Email.new(:name_of_template => name_of_template, :subject => subject, :text => text)
        result = new_email.save
        new_email = Email.last()
        reply = {"id" => new_email.id}
        return reply.to_json
      else
        email = Email.first(:id => id_template)
        email.update(:name_of_template => name_of_template)
        email.update(:text => text)
        result = email.update(:subject => subject)
        reply = "#{result}"
      end
      return reply.to_json
    end
  end

  #function for creating new user
  post '/save_user' do
    user = current_user
    if !user.nil?
      data = JSON.parse(request.body.read, :symbolize_names => true)
      name = data[:name]
      password = data[:password]
      auth = data[:auth]
      u = User.new(:name => name, :password => password, :auth => auth)
      result = u.save
      reply = "#{result}"
      return reply.to_json
    end
  end

  #function for deleting email template
  post '/delete_template' do
    user = current_user
    if !user.nil?
      data = JSON.parse(request.body.read, :symbolize_names => true)
      id_template = data[:id]

      email = Email.first(:id => id_template)

      email.destroy
    end
  end

  #function which will change password
  post '/change_password' do
    user = current_user
    if !user.nil?
      data = JSON.parse(request.body.read, :symbolize_names => true)
      old_pass = data[:oldPass]
      new_pass = data[:newPass]
      new_pass_again = data[:newPassAgain]
      status = false
      if old_pass == current_user.password
        if new_pass == new_pass_again
          status = user.update(:password => new_pass)
        end
      end
      reply = "#{status}"
      return reply.to_json
    end
  end

  #function which will return all users
  get '/load_all_users' do
    user = current_user
    if !user.nil?
      users = User.all()
      users = users.to_json
      return users
    end

  end

  #function which will delete user
  post '/delete_user' do
    user = current_user
    if !user.nil?
      if user.auth == 'admin'
        users = User.all()
        users = users.to_json
        return users
      else
        reply = "false"
        return reply.to_json
      end
    end
  end

  #function which will return logged user
  get '/logged_user' do
    user = current_user
    if !user.nil?
      return user.to_json
    end
  end

  #function which will set invitation expiration 
  post '/set_expiration' do
    user = current_user
    if !user.nil?
      data = JSON.parse(request.body.read, :symbolize_names => true)
      value = data[:value]
      user = User.first(:name => session[:username])
      user = User.update(:expiration => value)
      reply = 'true'
      return reply.to_json
    end
  end
end

# vim:ff=unix ts=2 ss=2
